@@ -1,58 +1,180 @@
-
-# rc, parametric margins
-rc_copula_log_lik <-function(p, x1, x2,indata1,indata2, quantiles = NULL, copula, m.dist)
+# rc, cox margin
+rc_copula_log_lik_cox <-function(p, x1, x2, indata1, indata2, copula)
 {
 
-  if (m.dist == "Piecewise") {
+  if (copula != "Copula2") {
+    eta <- p[1]
+    beta <- p[2:length(p)] # coefficients
+  }
 
-    # Cut event obs_time into 4 parts
-    dat<-rbind(indata1,indata2)
-    n.cons<-length(quantiles)-1
+  if (copula == "Copula2") {
+    alpha <- p[1]
+    kappa <- p[2]
+    beta <- p[3:length(p)] # coefficients
+  }
 
-    if (copula != "Copula2") {
+  t1<-indata1[, "obs_time"]
+  t2<-indata2[, "obs_time"]
+  c1<-indata1[, "status"]
+  c2<-indata2[, "status"]
 
-      eta<-p[n.cons+1] # association
-      beta<-p[(n.cons+2):length(p)] # coefficients
+  t <- c(t1, t2)
+  c <- c(c1, c2)
+  x1 <- matrix(x1, nrow = length(t1))
+  x2 <- matrix(x2, nrow = length(t2))
+  x12 <- rbind(x1, x2)
+  tab <- data.frame(table(t[c == 1]))
+  y <- as.numeric(levels(tab[, 1]))[tab[, 1]] # sorted unique event times
+  d <- tab[, 2] # number of events at each unique time (in case of tied events)
 
-    }
+  Lambda <- 0
+  Lambda <- 0
 
-    if (copula == "Copula2") {
+  # Create hazard function and calculate observed cumulative hazard and instanenuous hazard
+  h0 <- d/sapply(y,
+                 function(x) {
+                   sum(exp(as.matrix(x12[t >= x, ]) %*% beta))
+                 })
+  lambda <- rowSums(sapply(y, function(x) t==x) %*% h0 * c)
+  # # new_c: ones who are censored but have the same observation times as event times
+  # new_c <- c
+  # new_c[which(t %in% y)] <- 1
+  # lambda <- rowSums(sapply(y, function(x) t==x) %*% h0 * new_c)
+  Lambda <- as.numeric(sapply(y, function(x) t>=x) %*% h0)
 
-      alpha<-p[(n.cons+1)] # association
-      kappa<-p[(n.cons+2)] # association
-      beta<-p[(n.cons+3):length(p)] # coefficients
+  lambda1 <- lambda[1:length(t1)]
+  lambda2 <- lambda[-(1:length(t1))]
+  Lambda1 <- Lambda[1:length(t1)]
+  Lambda2 <- Lambda[-(1:length(t1))]
 
-    }
+  # tab1 <- data.frame(table(t1[c1 == 1]))
+  # y1 <- as.numeric(levels(tab1[, 1]))[tab1[, 1]] # sorted unique event times
+  # d1 <- tab1[, 2] # number of events at each unique time (in case of tied events)
+  #
+  # tab2 <- data.frame(table(t2[c2 == 1]))
+  # y2 <- as.numeric(levels(tab2[, 1]))[tab2[, 1]]
+  # d2 <- tab2[, 2]
+  #
+  # Lambda1 <- 0
+  # Lambda2 <- 0
+  # lambda1 <- 0
+  # lambda2 <- 0
+  #
+  # # Create hazard function and calculate observed cumulative hazard and instanenuous hazard
+  # h01<-d1/sapply(y1,
+  #                function(x) {
+  #                  sum(exp(as.matrix(x1[t1 >= x, ]) %*% beta))
+  #                })
+  # lambda1 <- rowSums(sapply(y1, function(x) t1==x) %*% h01 * c1)
+  # Lambda1 <- as.numeric(sapply(y1, function(x) t1>=x) %*% h01)
+  #
+  # h02 <- d2/sapply(y2,
+  #                  function(x) {
+  #                    sum(exp(as.matrix(x1[t2 >= x, ]) %*% beta))
+  #                  })
+  # lambda2 <- rowSums(sapply(y2, function(x) t2 == x) %*% h02 * c2)
+  # Lambda2 <- as.numeric(sapply(y2, function(x) t2 >= x) %*% h02)
 
-    t1<-indata1[,"obs_time"]
-    t2<-indata2[,"obs_time"]
-    # x1<-as.matrix(indata1[,var_list])
-    # x2<-as.matrix(indata2[,var_list])
+  # survival and density
+  u1 <- exp(-Lambda1 * exp(x1 %*% beta))
+  u2 <- exp(-Lambda2 * exp(x2 %*% beta))
+  u1_t1 <- -u1 * lambda1 * exp(x1 %*% beta)
+  u2_t2 <- -u2 * lambda2 * exp(x2 %*% beta)
 
-    Lambda1<-0 # left eye
-    Lambda2<-0 # right eye
+  ### copula models ######
+  if (copula == "Copula2") {
 
-    lambda1<-0 # left eye
-    lambda2<-0 # right eye
-
-    for (i in 1:n.cons)
-    {
-      Lambda1<-Lambda1+ p[i] * apply(cbind(rep(0,length(t1)), apply(cbind(rep(quantiles[i+1],length(t1)),t1),1,min) - quantiles[i]) ,1, max)
-      lambda1<-lambda1+p[i]*(t1<=quantiles[i+1])*(t1>quantiles[i])
-
-      Lambda2<-Lambda2+p[i]*apply(cbind(rep(0,length(t2)),apply(cbind(rep(quantiles[i+1],length(t2)),t2),1,min)-quantiles[i]),1,max)
-      lambda2<-lambda2+p[i]*(t2<=quantiles[i+1])*(t2>quantiles[i])
-    }
-
-    u1<-exp(-Lambda1*exp(x1%*%beta))
-    u2<-exp(-Lambda2*exp(x2%*%beta))
-    u1_t1<- -u1*lambda1*exp(x1%*%beta)
-    u2_t2<- -u2*lambda2*exp(x2%*%beta)
+    # Copula2 Copula function for joint distribution probability
+    C_val <- (1 + ((u1^(-1/kappa)-1)^(1/alpha) + (u2^(-1/kappa)-1)^(1/alpha))^alpha)^(-kappa)
+    c_u1_val <- C_val^(1+1/kappa) * (1 + ((u2^(-1/kappa)-1)/(u1^(-1/kappa)-1))^(1/alpha) )^(alpha-1)  * u1^(-1/kappa-1) # wrt to u1
+    c_u2_val <- C_val^(1+1/kappa) * (1 + ((u1^(-1/kappa)-1)/(u2^(-1/kappa)-1))^(1/alpha) )^(alpha-1)  * u2^(-1/kappa-1) # wrt to u2
+    c_val <- (1+1/kappa) * C_val^(1/kappa) * c_u2_val * (1 + ((u2^(-1/kappa)-1)/(u1^(-1/kappa)-1))^(1/alpha) )^(alpha-1) * u1^(-1-1/kappa) +
+      (-1+1/alpha) * (1/kappa) * C_val^(1+1/kappa) * (u1*u2)^(-1-1/kappa) * (1 + ((u2^(-1/kappa)-1)/(u1^(-1/kappa)-1))^(1/alpha) )^(alpha-2) * (u2^(-1/kappa)-1)^(-1+1/alpha) * (u1^(-1/kappa)-1)^(-1/alpha)
 
   }
 
 
 
+  if (copula == "Joe") {
+
+    # Joe joint and density functions
+    C_val <- 1 - ((1-u1)^eta + (1-u2)^eta - ((1-u1)^eta)*((1-u2)^eta) )^(1/eta)
+    c_u1_val <- ((1-u1)^eta + (1-u2)^eta - ((1-u1)^eta)*((1-u2)^eta) )^(1/eta - 1)  *  ((1-u1)^(eta-1) - (1-u1)^(eta-1)*(1-u2)^eta)
+    c_u2_val <- ((1-u1)^eta + (1-u2)^eta - ((1-u1)^eta)*((1-u2)^eta) )^(1/eta - 1)  *  ((1-u2)^(eta-1) - (1-u2)^(eta-1)*(1-u1)^eta)
+    c_val <- ((1-u1)^eta + (1-u2)^eta - ((1-u1)^eta)*((1-u2)^eta) )^(1/eta - 2) * (1/eta - 1) * ((1-u1)^(eta-1) - (1-u1)^(eta-1)*(1-u2)^eta) *  ((1-u2)^(eta-1) - (1-u2)^(eta-1)*(1-u1)^eta) * (-eta)+
+      ((1-u1)^eta + (1-u2)^eta - ((1-u1)^eta)*((1-u2)^eta) )^(1/eta - 1) * (eta * ((1-u1)^(eta-1)) * ((1-u2)^(eta-1)) )
+
+  }
+
+
+  if (copula == "Gumbel") {
+
+    c_val<-((-log(u1))^(eta-1)*(-log(u2))^(eta-1)*gh_F(u1,u2,eta)*((-log(u1))^eta+(-log(u2))^eta)^(2/eta-2))/(u1*u2)-
+      gh_F(u1,u2,eta)*((-log(u1))^eta+(-log(u2))^eta)^(1/eta-2)*(1/(u1*u2))*(1/eta-1)*eta*(-log(u1))^(eta-1)*(-log(u2))^(eta-1)
+    c_u1_val<-gh_F(u1,u2,eta)*((-log(u1))^eta+(-log(u2))^eta)^(1/eta-1)*(-log(u1))^(eta-1)/u1
+    c_u2_val<-gh_F(u1,u2,eta)*((-log(u1))^eta+(-log(u2))^eta)^(1/eta-1)*(-log(u2))^(eta-1)/u2
+    C_val<-gh_F(u1,u2,eta)
+
+  }
+
+
+  if (copula == "AMH") {
+
+    # AHM joint and desity functions
+    c_val <- ((eta^2)*(1-u1)*(1-u2) + eta*(u1*u2+u1+u2-2) + 1)/(1-eta*(1-u1)*(1-u2))^3
+    c_u1_val<- u2*(1-eta*(1-u2))/(1-eta*(1-u1)*(1-u2))^2
+    c_u2_val<- u1*(1-eta*(1-u1))/(1-eta*(1-u1)*(1-u2))^2
+    C_val<- u1*u2/(1-eta*(1-u1)*(1-u2))
+
+  }
+
+
+  if (copula == "Frank") {
+
+    # Frank Copula function for joint distribution probability
+    c_val <- (-1) * eta*exp((-1) * eta*u1)*exp((-1) * eta*u2)/((exp((-1) * eta)-1)+(exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)) -
+      (-1) * eta*exp((-1) * eta*u1)*exp((-1) * eta*u2)*(exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/((exp((-1) * eta)-1)+(exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1))^2
+    c_u1_val <- (exp((-1) * eta*u2)-1)*exp((-1) * eta*u1)/((1 + (exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/(exp((-1) * eta)-1))*(exp((-1) * eta)-1))
+    c_u2_val <- (exp((-1) * eta*u1)-1)*exp((-1) * eta*u2)/((1 + (exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/(exp((-1) * eta)-1))*(exp((-1) * eta)-1))
+    C_val <- (1/(-1) * eta) * log(1 + (exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/(exp((-1) * eta)-1))
+
+  }
+
+
+  if (copula == "Clayton") {
+
+    # Clayton Copula function for joint distribution probability
+    c_val<-clt_f(u1,u2,eta)
+    c_u1_val<-u1^(-eta-1)*(u1^(-eta)+u2^(-eta)-1)^(-1/eta-1)
+    c_u2_val<-u2^(-eta-1)*(u1^(-eta)+u2^(-eta)-1)^(-1/eta-1)
+    C_val<-(u1^(-eta)+u2^(-eta)-1)^(-1/eta)
+
+  }
+
+
+  C_val <- ifelse((indata1[,"status"] == 0) & (indata2[,"status"] == 0), C_val, 1)
+  term1 <- log(abs(C_val))
+
+  term2 <- c_u1_val * (-u1_t1)
+  term2 <- ifelse((indata1[,"status"] == 1) & (indata2[,"status"] == 0), term2, 1)
+  term2 <- log(abs(term2))
+
+  term3 <- (c_u2_val) * (-u2_t2)
+  term3 <- ifelse((indata1[,"status"] == 0) & (indata2[,"status"] == 1), term3, 1)
+  term3 <- log(abs(term3))
+
+  term4 <- c_val * u1_t1 * u2_t2
+  term4 <- ifelse((indata1[,"status"] == 1) & (indata2[,"status"] == 1), term4, 1)
+  term4 <- log(abs(term4))
+
+  logL <- (-1) * sum(term1 + term2 + term3 + term4)
+  return(logL)
+}
+
+# rc, parametric margins
+rc_copula_log_lik <-function(p, x1, x2,indata1,indata2, quantiles = NULL, copula, m.dist)
+{
+
   if (m.dist == "Weibull") {
 
     lambda <- p[1]
@@ -201,11 +323,11 @@ rc_copula_log_lik <-function(p, x1, x2,indata1,indata2, quantiles = NULL, copula
   if (copula == "Frank") {
 
     # Frank Copula function for joint distribution probability
-    c_val <- eta*exp(eta*u1)*exp(eta*u2)/((exp(eta)-1)+(exp(eta*u1)-1)*(exp(eta*u2)-1)) -
-      eta*exp(eta*u1)*exp(eta*u2)*(exp(eta*u1)-1)*(exp(eta*u2)-1)/((exp(eta)-1)+(exp(eta*u1)-1)*(exp(eta*u2)-1))^2
-    c_u1_val <- (exp(eta*u2)-1)*exp(eta*u1)/((1 + (exp(eta*u1)-1)*(exp(eta*u2)-1)/(exp(eta)-1))*(exp(eta)-1))
-    c_u2_val <- (exp(eta*u1)-1)*exp(eta*u2)/((1 + (exp(eta*u1)-1)*(exp(eta*u2)-1)/(exp(eta)-1))*(exp(eta)-1))
-    C_val <- (1/eta) * log(1 + (exp(eta*u1)-1)*(exp(eta*u2)-1)/(exp(eta)-1))
+    c_val <- (-1) * eta*exp((-1) * eta*u1)*exp((-1) * eta*u2)/((exp((-1) * eta)-1)+(exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)) -
+      (-1) * eta*exp((-1) * eta*u1)*exp((-1) * eta*u2)*(exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/((exp((-1) * eta)-1)+(exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1))^2
+    c_u1_val <- (exp((-1) * eta*u2)-1)*exp((-1) * eta*u1)/((1 + (exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/(exp((-1) * eta)-1))*(exp((-1) * eta)-1))
+    c_u2_val <- (exp((-1) * eta*u1)-1)*exp((-1) * eta*u2)/((1 + (exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/(exp((-1) * eta)-1))*(exp((-1) * eta)-1))
+    C_val <- (1/(-1) * eta) * log(1 + (exp((-1) * eta*u1)-1)*(exp((-1) * eta*u2)-1)/(exp((-1) * eta)-1))
 
   }
 
@@ -332,10 +454,10 @@ ic_copula_log_lik_sieve <- function(para, p, m, x1, x2, bl1, br1, bl2, br2, inda
   if (copula == "Frank") {
 
     # Frank Copula function for joint distribution probability
-    C_val_1 <- 1/(eta) * log(1 + (exp(eta*u1_left)-1)*(exp(eta*u2_left)-1)/(exp(eta)-1))
-    C_val_2 <- 1/(eta) * log(1 + (exp(eta*u1_left)-1)*(exp(eta*u2_right)-1)/(exp(eta)-1))
-    C_val_3 <- 1/(eta) * log(1 + (exp(eta*u1_right)-1)*(exp(eta*u2_left)-1)/(exp(eta)-1))
-    C_val_4 <- 1/(eta) * log(1 + (exp(eta*u1_right)-1)*(exp(eta*u2_right)-1)/(exp(eta)-1))
+    C_val_1 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_left)-1)*(exp((-1) * eta*u2_left)-1)/(exp((-1) * eta)-1))
+    C_val_2 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_left)-1)*(exp((-1) * eta*u2_right)-1)/(exp((-1) * eta)-1))
+    C_val_3 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_right)-1)*(exp((-1) * eta*u2_left)-1)/(exp((-1) * eta)-1))
+    C_val_4 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_right)-1)*(exp((-1) * eta*u2_right)-1)/(exp((-1) * eta)-1))
 
   }
 
@@ -524,10 +646,10 @@ ic_copula_log_lik_param <- function(para, p, x1, x2, t1_left, t1_right, t2_left,
   if (copula == "Frank") {
 
     # Frank Copula function for joint distribution probability
-    C_val_1 <- 1/(eta) * log(1 + (exp(eta*u1_left)-1)*(exp(eta*u2_left)-1)/(exp(eta)-1))
-    C_val_2 <- 1/(eta) * log(1 + (exp(eta*u1_left)-1)*(exp(eta*u2_right)-1)/(exp(eta)-1))
-    C_val_3 <- 1/(eta) * log(1 + (exp(eta*u1_right)-1)*(exp(eta*u2_left)-1)/(exp(eta)-1))
-    C_val_4 <- 1/(eta) * log(1 + (exp(eta*u1_right)-1)*(exp(eta*u2_right)-1)/(exp(eta)-1))
+    C_val_1 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_left)-1)*(exp((-1) * eta*u2_left)-1)/(exp((-1) * eta)-1))
+    C_val_2 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_left)-1)*(exp((-1) * eta*u2_right)-1)/(exp((-1) * eta)-1))
+    C_val_3 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_right)-1)*(exp((-1) * eta*u2_left)-1)/(exp((-1) * eta)-1))
+    C_val_4 <- 1/((-1) * eta) * log(1 + (exp((-1) * eta*u1_right)-1)*(exp((-1) * eta*u2_right)-1)/(exp((-1) * eta)-1))
 
   }
 
